//
//  LLPinboardClient.h
//  Spillo
//
//  Created by Damien DeVille on 4/13/14.
//  Copyright (c) 2014 Damien DeVille. All rights reserved.
//

#import <Foundation/Foundation.h>

@class LLPinboardCredentials;

@interface LLPinboardClient : NSObject

/*!
	\brief
	The current credentials. Need to be set by the caller.
 */
@property (strong, nonatomic) LLPinboardCredentials *credentials;
extern NSString * const LLPinboardClientCredentialsKey;

#pragma mark - Authentication

/*!
	\brief
	Authenticates a user given a username and token.
	
	\param username
	The username for the given user account. Required.
	
	\param token
	The API token for the given user account. Required.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(LLPinboardCredentials *credentials, NSError *error)`.
 */
- (NSOperation *)authenticateWithUsername:(NSString *)username token:(NSString *)token;

/*!
	\brief
	Authenticates a user given a username and password.
	
	\param username
	The username for the given user account. Required.
	
	\param password
	The password for the given user account. Required.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(LLPinboardCredentials *credentials, NSError *error)`.
 */
- (NSOperation *)authenticateWithUsername:(NSString *)username password:(NSString *)password;

/*!
	\brief
	Retrieve the user secret feed key.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSString *secretFeedKey, NSError *error)`.
 */
- (NSOperation *)retrieveUserSecretFeedKey;

#pragma mark - Posts

/*!
	\brief
	Retrieves the most recent time a bookmark was added, updated or deleted.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSDate *updateDate, NSError *error)`.
 */
- (NSOperation *)retrieveLastPostUpdateDate;

/*!
	\brief
	Creates a post with the given properties.
	
	\param properties
	A dictionary of properties to apply to the post. The `url` and `title` properties are required.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSNumber *success, NSError *error)`.
 */
- (NSOperation *)createPostWithProperties:(NSDictionary *)properties;

/*!
	\brief
	Updates a post with the given properties.
	
	\param properties
	A dictionary of properties to apply to the post. The `url` and `title` properties are required.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSNumber *success, NSError *error)`.
 */
- (NSOperation *)updatePostWithProperties:(NSDictionary *)properties;

/*!
	\brief
	Delete a post for a given URL.
	
	\param properties
	A dictionary of properties for the deleted post. The `url` property is required.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSNumber *success, NSError *error)`.
 */
- (NSOperation *)deletePostWithProperties:(NSDictionary *)properties;

/*!
	\brief
	Retrieves a list of dates with the number of posts at each date.
	
	\param tags
	A list of tag titles. Optional. Maximum 3.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *dates, NSError *error)`.
 */
- (NSOperation *)retrievePostDatesWithTags:(NSSet *)tags;

/*!
	\brief
	Retrieves a list of recent posts.
	
	\param tags
	A list of tag titles. Optional. Maximum 3.
	
	\param count
	The max number of posts to retrieve. 100 maximum. If 0 is specified default to 15.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *posts, NSError *error)`.
 */
- (NSOperation *)retrieveRecentPostsWithTags:(NSSet *)tags count:(NSUInteger)count;

/*!
	\brief
	Retrieves a list of all posts.
	
	\param tags
	A list of tag titles. Optional. Maximum 3.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *posts, NSError *error)`.
 */
- (NSOperation *)retrieveAllPostsWithTags:(NSSet *)tags;

/*!
	\brief
	Retrieves a count of posts from an specific offset optionally for given tags.
	
	\param tags
	A list of tag titles. Optional. Maximum 3.
	
	\param offset
	The offset to retrieve from.
	
	\param count
	The number of posts to retrieve.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *posts, NSError *error)`.
 */
- (NSOperation *)retrieveAllPostsWithTags:(NSSet *)tags offset:(NSUInteger)offset count:(NSUInteger)count;

/*!
	\brief
	Retrieves a count of posts between 2 dates optionally for given tags.
	
	\param tags
	A list of tag titles. Optional. Maximum 3.
	
	\param startDate
	The start date to retrieve posts from.
	
	\param endDate
	The end date to retrieve posts from.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *posts, NSError *error)`.
 */
- (NSOperation *)retrieveAllPostsWithTags:(NSSet *)tags startDate:(NSDate *)startDate endDate:(NSDate *)endDate;

/*!
	\brief
	Retrieve one or more posts on a single day matching the tags and/or URL. If no date or url is given, date of most recent bookmark will be used.
	
	\param tags
	A list of tag titles. Optional. Maximum 3.
	
	\param URL
	The URL of a post to retrieve.
	
	\param date
	The date of the posts to retrieve. If no date (and no url) is provided, the today date will be used.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *posts, NSError *error)`.
 */
- (NSOperation *)retrievePostsWithTags:(NSSet *)tags URL:(NSString *)URL date:(NSDate *)date;

/*!
	\brief
	Retrieves a list of suggested tags for a given post URL.
	
	\param postURL
	The URL of the post for which we want suggested tags. Required.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *tags, NSError *error)`.
 */
- (NSOperation *)retrievePostSuggestedTagsForURL:(NSString *)postURL;

#pragma mark - Tags

/*!
	\brief
	Retrieves all the tags in the user account.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *tags, NSError *error)`.
 */
- (NSOperation *)retrieveTags;

/*!
	\brief
	Deletes a tag matching the given title.
 
	\param tag
	The title of the tag to delete.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSNumber *success, NSError *error)`.
 */
- (NSOperation *)deleteTag:(NSString *)tag;

/*!
	\brief
	Deletes a tag matching the given title.
	
	\param tag
	The title of the tag to delete.
	
	\param updatedTag
	The updated title of the tag.
 
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSNumber *success, NSError *error)`.
 */
- (NSOperation *)renameTag:(NSString *)tag updatedTag:(NSString *)updatedTag;

#pragma mark - Feeds

/*!
	\brief
	The default count for the number of bookmarks returned by a feed request. 200.
 */
extern const NSUInteger LLPinboardClientFeedDefaultResultsCount;

/*!
	\brief
	Retrieves a list of posts for a feed.
	
	\param method
	The method for a feed.
	
	\param authenticated
	YES if the feed request should be authenticated with the feed secret.
	
	\param resultsCount
	The number of bookmarks to return from the feed request.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *posts, NSError *error)`.
 */
- (NSOperation *)retrieveFeedWithMethod:(NSString *)method authenticated:(BOOL)authenticated resultsCount:(NSUInteger)resultsCount;

/*!
	\brief
	Retrieves a list of posts for a feed.
	
	\param source
	The source for a feed.
	
	\param resultsCount
	The number of bookmarks to return from the feed request.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *posts, NSError *error)`.
 */
- (NSOperation *)retrieveFeedWithSource:(NSString *)source resultsCount:(NSUInteger)resultsCount;

/*!
	\brief
	Retrieves a list of posts for a feed.
	
	\param username
	The username for a feed.
	
	\param tags
	A list of tags for a feed.
	
	\param resultsCount
	The number of bookmarks to return from the feed request.
	
	\return
	An operation to be enqueued with completion provider `void (^completionProvider)(NSArray *posts, NSError *error)`.
 */
- (NSOperation *)retrieveFeedWithUsername:(NSString *)username tags:(NSSet *)tags resultsCount:(NSUInteger)resultsCount;

@end

@interface NSOperation (LLPinboardClient)

@property (readonly, copy, atomic) id (^completionProvider)(NSError **errorRef);

@end
