//
//  LLSpilloDataAccountManager.h
//  Spillo
//
//  Created by Damien DeVille on 10/16/16.
//  Copyright © 2016 Damien DeVille. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <LLSpilloData/LLSpilloDataAccount.h>

typedef NS_OPTIONS(NSUInteger, LLSpilloDataAccountManagerType) {
	// A UI client is mostly interested in displaying data. Thus, it will not be able to refresh or check for dead links but it will create the default items.
	LLSpilloDataAccountManagerTypeUI = 1 << 0,
	// A sync client will refresh each account with the server automatically, check for dead links, etc...
	LLSpilloDataAccountManagerTypeSync = 1 << 1,
	// An extension client is neither syncing nor the main UI client. Thus, it won't either refresh or create any default items.
	LLSpilloDataAccountManagerTypeExtension = 1 << 2,
};

@class LLSpilloDataPersistentStoreNotification;

@interface LLSpilloDataAccountManagerConfiguration : NSObject

/*!
	\brief
	The folder in which the persistent stores for each account will be written.
 */
@property (copy, nonatomic) NSURL *storeContainerLocation;

/*!
	\brief
	An instance of `NSUserDefaults` to use to store the accounts
 */
@property (strong, nonatomic) NSUserDefaults *userDefaults;

/*!
	\brief
	The (optional) access group to use when storing and retrieving the account credentials.
 */
@property (copy, nonatomic) NSString *keychainAccessGroup;

@end

@class LLSpilloDataBookmarkEngine;

@interface LLSpilloDataAccountManager : NSObject

/*!
	\brief
	Instantiate a new account manager.
	
	\param type
	The account manager type
	
	\param configuration
	And instance of `LLSpilloDataAccountManagerConfiguration` containing the configuration for this account manager.
 */
- (instancetype)initWithType:(LLSpilloDataAccountManagerType)type configuration:(LLSpilloDataAccountManagerConfiguration *)configuration NS_DESIGNATED_INITIALIZER;

/*!
	\brief
	The type of account manager.
 */
@property (readonly, assign, nonatomic) LLSpilloDataAccountManagerType type;

/*!
	\brief
	All the accounts currently registered in the account manager. Should only be called on the main thread. Observable.
 */
@property (readonly, strong, nonatomic) NSArray<LLSpilloDataAccount *> *accounts;
extern NSString * const LLSpilloDataAccountManagerAccountsKey;

/*!
	\brief
	Create and add an account for the given type. Should only be called on the main thread.
 */
- (LLSpilloDataAccount *)addAccountWithType:(LLSpilloDataAccountType)type error:(NSError **)errorRef;

/*!
	\brief
	Remove the given account from the manager. Should only be called on the main thread.
 */
- (BOOL)removeAccount:(LLSpilloDataAccount *)account error:(NSError **)errorRef;

/*!
	\brief
	Reload all the accounts from disk. Can be useful if another process has mutated accounts and the current process
	wants to reload them all.
 */
- (void)reloadAccountsFromDisk;

/*!
	\brief
	Returns the bookmark engine associated with this account. Should only be called on the main thread.
 */
- (LLSpilloDataBookmarkEngine *)bookmarkEngineForAccount:(LLSpilloDataAccount *)account;

/*!
	\brief
	The accounts that are currently authenticted but not necessarily loaded. Should only be called on the main thread. Observable.
 */
@property (readonly, strong, nonatomic) NSArray<LLSpilloDataAccount *> *authenticatedAccounts;
extern NSString * const LLSpilloDataAccountManagerAuthenticatedAccountsKey;

/*!
	\brief
	The accounts that are currently loaded (i.e. have their store setup) and authenticated. Should only be called on the main thread. Observable.
 */
@property (readonly, strong, nonatomic) NSArray<LLSpilloDataAccount *> *loadedAccounts;
extern NSString * const LLSpilloDataAccountManagerLoadedAccountsKey;

/*!
	\brief
	Returns the account matching the given identifier. Should only be called on the main thread.
 */
- (LLSpilloDataAccount *)accountWithIdentifier:(NSString *)identifier;

/*!
	\brief
	Process a persistent store notification coming from another process and merge it into the local contexts.
	
	\param persistentStoreNotification
	The persistent store notification to process.
 */
- (void)processPersistentStoreNotification:(LLSpilloDataPersistentStoreNotification *)persistentStoreNotification;

@end

/*!
	\brief
	Notification posted when the accounts were persisted to disk. Can be used in order to message other processes that they should refetch their accounts from disk.
 */
extern NSString * const LLSpilloDataAccountManagerDidPersistAccountsNotification;

/*!
	\brief
	Notification posted whenever an account bookmark engine persists some data. Can be used to message other processes so that they refresh their state from disk.
 */
extern NSString * const LLSpilloDataAccountManagerDidUpdatePersistentStoreNotification;
extern NSString * const LLSpilloDataAccountManagerDidUpdatePersistentStoreNotificationKey;
