//
//  LLSpilloDataBookmarkEngine+Syncing.h
//  Spillo
//
//  Created by Damien DeVille on 10/16/16.
//  Copyright © 2016 Damien DeVille. All rights reserved.
//

#import <LLSpilloData/LLSpilloDataBookmarkEngine.h>

@class LLSpilloPinboardPost;

@interface LLSpilloDataBookmarkEngine (Syncing)

#pragma mark - Authentication

/*!
	\brief
	Start authentication for a given authentication context.
	
	\param context
	An opaque context containing information to authenticate. Required.
	
	\param completionHandler
	A block that will be invoked when the authentication completes or fails.
 */
- (void)authenticateWithContext:(id)context completionHandler:(void (^)(BOOL authenticated, NSError *error))completionHandler;

/*!
	\brief
	Log out the current user from the sync client and delete the local cache.
	
	\param errorRef
	An error returned by reference.
 */
- (BOOL)logout:(NSError **)errorRef;

#pragma mark - Refresh

/*!
	\brief
	YES if the controller is currently refreshing, NO otherwise. Observable. Changes on the main thread.
 */
@property (readonly, getter = isRefreshing, assign, atomic) BOOL refreshing;
extern NSString * const LLSpilloDataBookmarkEngineRefreshingKey;

/*!
	\brief
	A block that is invoked whenever refresh completes.
 */
@property (copy, nonatomic) void (^refreshCompletion)(BOOL refreshed, NSError *error);
extern NSString * const LLSpilloDataBookmarkEngineRefreshCompletionKey;

/*!
	\brief
	Start refreshing the user's data.
	
	\param forced
	If YES, the bookmark engine will force a refresh, even if there's nothing to refresh.
	
	\return
	A cancellable token.
 */
- (id <LLSpilloDataToken>)refresh:(BOOL)forced;

/*!
	\brief
	Start refreshing the user's data, only sending the local changes rather than doing a full refresh.
	
	\param completionHandler
	A block that will be invoked when the refresh completes or fails.
 
	\return
	A cancellable token.
 */
- (id <LLSpilloDataToken>)sendLocalChanges:(void (^)(BOOL refreshed, NSError *error))completionHandler;

#pragma mark - Search

/*!
	\brief
	Start retrieving a list of posts for a given search.
	
	\param search
	The search to be performed.
	
	\param completionHandler
	A block that will be invoked when the search completes or fails.
	
	\return
	A cancellable token.
 */
- (id <LLSpilloDataToken>)retrievePostsForSearch:(LLSpilloSearch *)search completionHandler:(void (^)(NSArray *posts, NSError *error))completionHandler;

/*!
	\brief
	Start retrieving a list of suggested tags for a given url.
	
	\param postURL
	The URL to retrieve suggested tags for.
	
	\param completionHandler
	A block that will be invoked when the retrieval completes or fails.
	
	\return
	A cancellable token.
 */
- (id <LLSpilloDataToken>)retrieveSuggestedTagsForURL:(NSString *)postURL completionHandler:(void (^)(NSArray *suggestedTags, NSError *error))completionHandler;

@end
