//
//  LLSpilloDataBookmarkEngine.h
//  Spillo
//
//  Created by Damien DeVille on 4/14/14.
//  Copyright (c) 2014 Damien DeVille. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreData/CoreData.h>

@class LLSpilloDataAccount, LLSpilloSearch, LLSpilloPinboardPost;
@protocol LLSpilloDataToken;

@interface LLSpilloDataBookmarkEngine : NSObject

/*!
	\brief
	The account associated with the data controller.
 */
@property (readonly, strong, nonatomic) LLSpilloDataAccount *account;

#pragma mark - Store

/*!
	\brief
	Setup the store and return an error if it failed.
 */
- (BOOL)setupStore:(NSError **)errorRef;

/*!
	\brief
	Teardown the store and return an error if it failed.
 */
- (BOOL)teardownStore:(NSError **)errorRef;

/*!
	\brief
	A managed object context of concurrency type `NSMainQueueConcurrencyType` tied to the data store.
	Will be nil when the account is not loaded. The account can be loaded manually by calling `setupStore`.
 */
@property (readonly, strong, nonatomic) NSManagedObjectContext *mainManagedObjectContext;
extern NSString * const LLSpilloDataBookmarkEngineMainManagedObjectContextKey;

#pragma mark - Status

/*!
	\brief
	YES if the controller is currently doing some work, NO otherwise. Observable. Changes on the main thread.
 */
@property (readonly, getter = isWorking, assign, nonatomic) BOOL working;
extern NSString * const LLSpilloDataBookmarkEngineWorkingKey;

/*!
	\brief
	The current work status. Observable. Changes on the main thread.
 */
@property (readonly, copy, nonatomic) NSString *status;
extern NSString * const LLSpilloDataBookmarkEngineStatusKey;

#pragma mark - Link checker

/*!
	\brief
	YES if the controller is currently checking dead links, NO otherwise. Observable. Changes on the main thread.
 */
@property (readonly, getter = isCheckingDeadLinks, assign, atomic) BOOL checkingDeadLinks;
extern NSString * const LLSpilloDataBookmarkEngineCheckingDeadLinksKey;

/*!
	\brief
	Start looking for dead links.
	
	\param completionHandler
	A block that will be invoked when the check completes or fails.
 */
- (id <LLSpilloDataToken>)checkDeadLinks:(void (^)(BOOL completed, NSError *error))completionHandler;

#pragma mark - Deletion

/*!
	\brief
	Cancel all work currently being done by the bookmark engine.
 */
- (void)cancelWork;

/*!
	\brief
	Prepare for deletion, returning YES if the bookmark engine is ready to be deleted.
	Note that the engine cannot be used any longer after this method has been called and returned YES.
 */
- (BOOL)prepareForDeletion;

@end

extern NSString * const LLSpilloDataBookmarkEnginePersistentStoreDidUpdateNotification;
extern NSString * const LLSpilloDataBookmarkEnginePersistentStoreDidUpdateNotificationKey;
