//
//  LLSpilloDataBookmarkSyncClient.h
//  Spillo
//
//  Created by Damien DeVille on 10/15/16.
//  Copyright © 2016 Damien DeVille. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreData/CoreData.h>

@class LLSpilloSearch;
@protocol LLSpilloDataToken;

@protocol LLSpilloDataBookmarkSyncClient <NSObject>

@required

/*!
	\brief
	Instantiates a new data sync client.
	
	\param managedObjectContext
	A managed object context to be used for syncing. Should be of `NSPrivateQueueConcurrencyType` type.
	
	\param accountIdentifier
	The identifier for the current account.
	
	\param keychainAccessGroup
	The access group to use when retrieving and storing in the keychain.
 */
- (instancetype)initWithManagedObjectContext:(NSManagedObjectContext *)managedObjectContext accountIdentifier:(NSString *)accountIdentifier keychainAccessGroup:(NSString *)keychainAccessGroup;

#pragma mark - Authentication

/*!
	\brief
	Log in the give user.
 
	\param username
	The username for the current user.
 
	\param errorRef
	A reference to an error.
	
	\return
	YES if the sync client correctly logged in, NO otherwise.
 */
- (BOOL)login:(NSString *)username error:(NSError **)errorRef;

/*!
	\brief
	Log out the current user from the sync client.
 
	\param errorRef
	A reference to an error.
	
	\return
	YES if the sync client correctly logged out, NO otherwise.
 */
- (BOOL)logout:(NSError **)errorRef;

/*!
	\brief
	Start authentication for a given authentication context.
	
	\param context
	An opaque context containing information to authenticate. Required.
	
	\param completionHandler
	A block that will be invoked when the authentication completes or fails.
 */
- (void)authenticateWithContext:(id)context completionHandler:(void (^)(BOOL authenticated, NSString *username, NSError *error))completionHandler;

#pragma mark - Refreshing

/*!
	\brief
	YES if the sync client is currently refreshing, NO otherwise. Observable. Changes on the main thread.
 */
@property (readonly, assign, getter = isRefreshing, nonatomic) BOOL refreshing;
extern NSString * const LLSpilloDataBookmarkSyncClientRefreshingKey;

/*!
	\brief
	The current refresh status. Observable. Changes on the main thread.
 */
@property (readonly, copy, nonatomic) NSString *refreshStatus;
extern NSString * const LLSpilloDataBookmarkSyncClientRefreshStatusKey;

/*!
	\brief
	Start refreshing the user's data.
	
	\param localOnly
	If YES, only the local changes will be sent to the server, otherwise, the server changes will be pulled too.
 
	\param forced
	If YES, the bookmark engine will force a refresh, even if there's nothing to refresh.
 
	\param completionHandler
	A block that will be invoked when the refresh completes or fails.
 */
- (id <LLSpilloDataToken>)refreshLocalOnly:(BOOL)localOnly forced:(BOOL)forced completionHandler:(void (^)(BOOL refreshed, NSError *error))completionHandler;

@optional

#pragma mark - Search and suggestions

/*!
	\brief
	Start retrieving a list of posts for a given search.
	
	\param search
	The search to be performed.
	
	\param completionHandler
	A block that will be invoked when the search completes or fails.
	
	\return
	A token that can be used to cancel the search by calling `-cancelOperation:`.
 */
- (id <LLSpilloDataToken>)retrievePostsForSearch:(LLSpilloSearch *)search completionHandler:(void (^)(NSArray *posts, NSError *error))completionHandler;

/*!
	\brief
	Start retrieving a list of suggested tags for a given url.
	
	\param postURL
	The URL to retrieve suggested tags for.
	
	\param completionHandler
	A block that will be invoked when the retrieval completes or fails.
	
	\return
	A token that can be used to cancel the search by calling `-cancelOperation:`.
 */
- (id <LLSpilloDataToken>)retrieveSuggestedTagsForURL:(NSString *)postURL completionHandler:(void (^)(NSArray *suggestedTags, NSError *error))completionHandler;

@end
